//Paul S. Underwood
//SW409 Final
//May 3, 2005

//Borrowed from chapter 44

package addbk.JAddressBook;

import addbk.print.labels.LabelBean;
import utils.StringWacker;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;

public class PrintableLabel
        implements Printable {
    private Graphics2D g;

    private static final double POINTS_PER_MILLIMETER = 2.834;
    private static final int INCH = 72; //points per inch
    private final int labelMargin = INCH / 4;

    private AddressDataBase adb;
    private LabelBean lb = LabelBean.restore();

    private Dimension labelSize =
            new Dimension(
                    mm2Points(lb.getLabelWidthInMM()),
                    mm2Points(lb.getLabelHeightInMM()));

    final int internalLabelWidth = labelSize.width - labelMargin;


    /*
    There are 25.4 mms/inch
    There are 72 picas / inch
    There are 72/25.4 = 2.834 picas / mm
    */
    private Font font;

    public int mm2Points(int mm) {
        return (int) (mm * POINTS_PER_MILLIMETER);
    }

    /**
     * @param row    - number of row, starting at zero
     * @param column - number of column, starting at zero
     * @return upper left corner of label
     */
    public Point getLabelLocationInPoints(int row, int column) {
        final Point pageMargin = new Point(mm2Points(lb.getPageMarginLeftInMM()),
                mm2Points(lb.getPageMarginTopInMM()));
        final int horizontalGutterInPoints = mm2Points(lb.getHorizontalGutterInMM());
        final int verticalGutterInPoints = mm2Points(lb.getVerticalGutterInMM());
        Point p = new Point();
        p.x = pageMargin.x + (labelSize.width + horizontalGutterInPoints) * column;
        p.y = pageMargin.y + (labelSize.height + verticalGutterInPoints) * row;
        return p;
    }

    public PrintableLabel(AddressDataBase adb, Font font) {
        this.adb = adb;
        this.font = font;
    }

    //-----------
    //called by printing system
    //note that this simplistic example only prints page 0
    public int print(Graphics gr, PageFormat pageFormat, int page)
            throws PrinterException {
        this.g = (Graphics2D) gr;
        return printPage(page);
    }

    private int printPage(int page) {
        final int n = lb.getNumberOfLabelsPerSheet();
        int recordNumber = page * n;
        adb.setRecordNumber(recordNumber);
        AddressRecord[] ar = adb.getNextRecords(n);
        if (recordNumber > adb.getSize()) return NO_SUCH_PAGE;
        int row = 0;
        int column = 0;

        for (int i = 0; i < n; i++) {
            printOneLabel(ar[i], row, column);
            row++;
            if (row >= lb.getNumberOfLabelsPerColumn()) {
                column++;
                row = 0;
            }
        }
        return PAGE_EXISTS;
    }


    private String getShortString(String str) {

        StringWacker sr = new StringWacker(internalLabelWidth, str, font, g);
        return sr.getShortString();
    }


    //Break the given string into strings (space delimited)
    //which will fit on the given label with the given
    //font.
    private Lines substringify(String str) {
        Lines v = new Lines();
        String sub = str;
        Rectangle2D rect = font.getStringBounds(str,
                0,
                str.length(),
                g.getFontRenderContext());
        while (rect.getWidth() > internalLabelWidth) {
            String temp = getShortString(sub);
            v.add(temp);
            sub = sub.substring(temp.length());
            rect = font.getStringBounds(sub,
                    0,
                    sub.length(),
                    g.getFontRenderContext());
        }
        v.add(sub);
        return v;
    }

    private void printOneLabel(AddressRecord ar, int row, int column) {

        final Point p = getLabelLocationInPoints(row, column);
        //uncomment the following if labels are out of alignment
        //drawOutline(p);
        if (ar == null) return;
        //----
        g.setColor(Color.BLACK);

        String s[] = ar.getRecordAsSingleArray();
        g.setFont(font);
        int fontHeightInPicas = font.getSize();
        Lines allShortStrings = new Lines();
        Lines fieldShortStrings = new Lines();

        //Mailing labels built from Name and Address fields
        //(AddressRecord: 1st two fields)
        for (int i = 0; i < 2; i++) {
            //Process each display line (e.g. street, city/state)
            //separately
            String[] temp = s[i].split("\n");
            for (int k = 0; k < temp.length; ++k) {
                //Break field into label-width chunks
                fieldShortStrings.set(substringify(temp[k]));
                for (int ss = 0; ss < fieldShortStrings.size(); ++ss) {
                    //Add chunks to complete address vector
                    allShortStrings.add(fieldShortStrings.elementAt(ss));
                }
            }
        }

        int termCount = allShortStrings.size();
        if (termCount * fontHeightInPicas > labelSize.height) {
            StringBuffer al = new StringBuffer();
            for (int ll = 0; ll < allShortStrings.size(); ++ll) {
                al.append(allShortStrings.elementAt(ll));
                al.append('\n');
            }
            al.append("Address will not fit label!");
            al.append("\nChoose smaller font or larger label");
            System.out.println(al.toString());
        }

        for (int j = 0; j < termCount; ++j) {
            int localX = p.x;
            int localY = p.y + (j + 1) * fontHeightInPicas;
            String currentString = allShortStrings.elementAt(j);
            Rectangle2D rect = font.getStringBounds(currentString,
                    0,
                    currentString.length(),
                    g.getFontRenderContext());
            double stringWidthInPicas = rect.getWidth();
            double whiteSpace = labelSize.width - stringWidthInPicas;
            localX += (int) whiteSpace / 2;
            double textHeightInPicas = allShortStrings.size() * fontHeightInPicas;
            whiteSpace = labelSize.height - textHeightInPicas;
            localY += (int) whiteSpace / 2;
            g.drawString(currentString, localX, localY);
        }
        fieldShortStrings.removeAllElements();


    }

    /**
     * draws a light gray rectangle as an outline around the label.
     *
     * @param p - location of the label, computed from row and column and
     *          label dimensions.
     */
    private void drawOutline(Point p) {
        g.setColor(Color.LIGHT_GRAY);

        g.drawRect(p.x, p.y,
                labelSize.width, labelSize.height);
    }
}
